// 用于为海康球机PTZ值转换的函数
#pragma once
#include "DecisionPlatform.h"
#include <cmath>
#include <string>

typedef unsigned char BYTE;
typedef unsigned short WORD;

float HEX2DEC(WORD Hex);


WORD DEC2HEX(float Dec);

std::string Hex2String(const std::string &hex);

std::string PrepareHexString(std::string str);

// 十进制转十六进制
std::string decimalToHexadecimal(int decimal);
// 十六进制转十进制
int hexadecimalToDecimal(const std::string& hexadecimal);

std::string charToHex(char value);

std::string verify(const std::string& hexStr);